;Comment	~
;	x2b2.asm
;	A replacement for the Exe2Bin program, which is NOT included with
;	PC-DOS 3.3.
;	written by Henry T. Nettles, Dec. 21, 1987 -- Feb. 20, 1988
;	compiled with MicroSoft Macro Assembler, version 5.0
;	The inspiration (and model) for this program was EXE2COM.C, written
;	by Chris Dunford.
;	The include module, DOS.INC, comes with MASM 5.0
;	The subroutine BinToStr is borrowed from the SHOW.ASM program which
;	also comes with MASM 5.0
;	Some of this code (the command line parser) is borrowed from a program
;	by Vernon Buerg (TABS.ASM)
;	This program is hereby released to the public domain.
;
;	Usage:  >X2B2  [d:][\path\]PROG[.EXE]  [d:][\path\][PROG][.COM]
;
;	The only necessary parameter is the file name of the input file.  A
;	drive and/or path may be given if needed.
;	If no extension is given, .EXE is assumed.  If no ouput file is
;	given, then the same file name with an extension of .COM is used.
;	WARNING: If a drive and/or path is given for the input file, and
;	no ouput file is given, then the input drive and/or path will be
;	used for the ouput (not the default directory).
;
;	This program works on my computer, on the files I have tried it on.
;	However, extensive testing has not been performed, and I assume no
;	responsibility whatever for the program.  I would be interested in
;	bug reports, and if anyone improves the code it would be nice to
;	receive a copy.  I can be reached at:
;
;			22547 Braken Carter
;			Katy, Texas 77449-3619
;
;Toad Hall Tweak, 15 Oct 89
;- Converted to .COM format
;- General tightening
;- Moved EXE header buffer, filename buffers to dynamic space at code end
;  (to reduce program size)
;- Changed BinToStr procedure so it carries parms directly in AX/DI
;  instead of the slow pushes.
;- Removed some unnecessary local variables
;
;Comment	ends	~

RECSIZE		equ	8192
PATHNAMESIZE	equ	128 + 16
STACKSIZE	equ	512
CMDTAIL		equ	80h
CR		equ	0dh
LF		equ	0ah
%DEFINE	EOM	'$'
%define eom	EOM
SIZE_OF_HEADER	equ	28

struc exe_header		; what an exe header looks like
.exe_sig1:	resb	1	; EXE file signature: "MZ"
.exe_sig2:	resb	1
.excess:	resw	1	; image size mod 512 (valid bytes in last page)
.pages:		resw	1	; # of 512 byte pages in image
.relo_ct:	resw	1	; count of relocation table entries
.hdr_size:	resw	1	; size of header in paragraphs
.min_mem:	resw	1	; min required memory
.max_mem:	resw	1	; max required memory
.xss:		resw	1	; stack seg offset in load module
.xsp:		resw	1	; initial value of sp
.cksum:		resw	1	; file checksum
.xip:		resw	1	; initial value of IP
.xcs:		resw	1	; cs offset in load module
.relo_start:	resw	1	; offset of first relocatable item
.ovl_num:	resw	1	; overlay number
endstruc ; exe_header		; end of structure

SEGMENT	CSEG	ALIGN=16

	org	100H

X2B2:
	jmp	Start		;skip over runtime data			v1.1


errflag		db	0	;error flag				v1.1
ignore_stack:	db 0
ignore_reloc:	db 0
ignore_ip:	db 0
big_enable:	db 0
eof_enable:	db 0
	align 2, db 0
save_ip		dw	0
handle1		dw	0
handle2		dw	0
options:
	dw "S", setflag, ignore_stack
	dw "R", setflag, ignore_reloc
	dw "I", setflag, ignore_ip
	dw "B", setflag, big_enable
	dw "E", setflag, eof_enable
	dw "H", writehelp, 0
	dw "?", writehelp, 0
	dw 0

filerecords:
.1:
	dw filenamesbuffer
	dw 0
	dw msg1
.2:
	dw 0, 0, msg2
.end:
	dw 0, 0, msg3

	align 4, db 0
code_size:	dd 0		; amount of code to move to .com file
code_start:	dd 0


msg1	db	'Input File ==> "',eom
msg2	db	'"  Output File ==> "',eom
msg3	db	'"',CR,LF,EOM

LENGTHNUMBER equ 10

author		db	'X2B2 ecm release 0  -  Public Domain Software by Henry T. Nettles'
		db	CR,LF,EOM
code_size_msg	db	'Code Size = '
code_size_str:
		times LENGTHNUMBER db 32
		db EOM
code_start_msg	db	'  Code Start = '
code_start_str:
		times LENGTHNUMBER db 32
		db EOM
ip_msg		db	'  Initial IP = '
ip_str:
		times LENGTHNUMBER db 32
		db CR,LF,EOM
actual_code_size_msg db	'Size of output file = '
actual_code_size_str:
		times LENGTHNUMBER db 32
		db CR,LF,EOM

no_file_msg1:
	db "USAGE:  >X2B2 [switches] [d:][\path\]file[.exe] [d:][\path\][file][.com]", CR, LF
	db CR, LF
	db "Supported switches:", CR, LF
	db "  -I    Ignore IP in header, if it is neither 0 nor 100h assume 0", CR, LF
	db "  -S    Ignore stack in header", CR, LF
	db "  -R    Ignore MZ relocations present", CR, LF
	db "  -B    Allow image larger than nearly 64 KiB limit for .COM file", CR, LF
	db "  -E    Allow image to end before header indicated size", CR, LF
	db "  --    End of switches, next dash is part of filename", CR, LF
	db "  -H    Display help and exit", CR, LF
	db "  -?    Display help and exit", CR, LF
	db EOM
open_fail_msg  db 'ERROR: Open of input file failed!',CR,LF,EOM
create_fail_msg	db 'ERROR: Create of output file failed!',CR,LF,EOM
disk_full_msg  db 'ERROR: The disk is full!',CR,LF,EOM
bad_read_msg   db 'I/O Error on Read!',CR,LF,EOM
bad_write_msg  db 'I/O Error on Write!',CR,LF,EOM
seek_error_msg db 'I/O Error on Seek!',CR,LF,EOM

badsig_msg     db 'Invalid EXE File Signature',CR,LF,EOM
hasrelo_msg    db 'EXE has relocatable items',CR,LF,EOM
has_ss_msg     db 'EXE has stack segment',CR,LF,EOM
bad_ip_msg     db 'IP not 0 or 100h',CR,LF,EOM
too_big_msg    db 'EXE file is too big to convert to COM file. May want -B switch?',CR,LF,EOM
msg:
.trailing_garbage:
		db "ERROR: Trailing garbage on command line!", CR, LF, EOM
.wrongswitch:	db "ERROR: Wrong switch specified!", CR, LF, EOM
.shortreadhead:	db "ERROR: File too short to hold MZ header!", CR, LF, EOM
.shortreaddata:	db "ERROR: Short read during data read, file is too short.", CR, LF, EOM
.headererror:	db "ERROR: Header error!", CR, LF, EOM
.badstack:	db "ERROR: X2B2 stack is too small!", CR, LF, EOM


Start:					;v1.1

;****************************************************************************
;*	Get two file names from command line
;*	This portion of code was borrowed from TABS.ASM by Vernon Buerg
;****************************************************************************

	mov	dx,author	;tell them who we are		v1.1
	mov	ah,9			;display string
	int	21H

	cmp sp, stacktop
	jae goodstack
	mov dx, msg.badstack
	jmp Print_Error

goodstack:

	mov	si, CMDTAIL + 1		; DS:SI points to command line
	mov byte [0FFh - (CMDTAIL + 1) + si], 13
					; ensure command line tail terminated

getoptions:
	mov bx, "-/"
.:
	lodsb
	cmp al, 32
	je .
	cmp al, 9
	je .
	cmp al, 13
	je .done
	cmp al, bl
	je .switch
	cmp al, bh
	je .switch_xchg
.done:
	jmp getfilenames

.switch_xchg:
	xchg bl, bh
.switch:
	mov ah, al
	lodsb
	cmp al, ah
	jne .normalswitch
	mov bl, 13
	jmp .

.wrongswitch:
	jmp wrongswitch

.normalswitch:
	cmp al, 'a'
	jb .gotcap
	cmp al, 'z'
	ja .gotcap
	xor al, 'A' ^ 'a'
.gotcap:
	push si
	mov si, options - 6
.loopswitch:
	lea si, [si + 6]
	cmp byte [si], 0
	je .wrongswitch
	cmp byte [si], al
	jne .loopswitch
.gotswitch:
	pop ax
	call near [si + 2]
	xchg si, ax
	jmp .


setflag:
	mov si, word [si + 4]
	mov byte [si], -1
	retn


getfilenames:
	mov bp, filerecords.1		; -> first record
	mov di, word [bp]		; -> buffer
	db 0A8h				; skip lodsb (test al, imm8)
.loopblank:
	lodsb
	cmp al, 32
	je .loopblank
	cmp al, 9
	je .loopblank			; skip blanks
	cmp al, 13
	je .end				; if end -->

.nonblank:
	cmp bp, filerecords.end		; already have two pathnames ?
	mov dx, msg.trailing_garbage
	jae Print_Error			; yes -->

	mov word [bp], di		; -> pathname in buffer
	db 0A9h				; skip stosb, lodsb (test al, imm16)
.loopnonblank:
	stosb				; store nonblank text
	lodsb				; get next text
	cmp al, 32			; is it a blank or EOL ?
	je .donenonblank
	cmp al, 9
	je .donenonblank
	cmp al, 13
	jne .loopnonblank		; no, store text -->

.donenonblank:
	dec si				; -> blank or EOL
	call .finish			; store info
	jmp .loopblank			; return to loop

.finish:
	mov al, 0			; store a NUL terminator
	mov word [bp + 2], di		; -> where NUL terminator goes
	stosb				; store it
	add di, 4 + 1			; leave space for extension ".EEE"
	and di, ~1
	add bp, 6			; -> next record
	retn

.end:
	call .finish			; finish the last record


;***************************************************************************
;*	DISPLAY THE TWO FILE NAMES FROM COMMAND LINE
;***************************************************************************
	mov si, [filerecords.1]
	xor	al,al			;handy 0			v1.1
	cmp	al,[si]			;did user enter at least 1 file name? v1.1
	jnz	L1			; is ok, we have a file name
	 jmp	No_File1

L1:
	mov dx, [filerecords.1 + 2]
	inc dx
	sub dx, si			; = length including NUL
	mov cx, dx

	mov	di,si	;offset fname1	;point back to fname1 start	v1.1
	mov	al,'.'
	repnz	scasb			; search for period in file name
	jz	L1a			; found the period, must have ext

	mov	ax,si	;offset fname1	; no period, must add .EXE for user v1.1
	add	ax,dx	;fname1_len	;				v1.1
	dec	ax			;
	mov	di,ax			; di points to '\0' at end of file name
	mov	ax,'.E'	;452eh		; ".E" backwards		v1.1
	stosw
	mov	ax,'XE'	;4558h		; "XE" backwards		v1.1
	stosw
	mov word [filerecords.1 + 2], di
	mov al, 00h
	stosb

L1a:
	mov	cx,-1			;common code			v1.1
	mov si, [filerecords.2]
	test si, si
	jnz .havetwo

	mov si, [filerecords.1 + 2]
	add si, 1 + 1			; skip space for NUL
	and si, ~1
	mov word [filerecords.2], si
	mov word [filerecords.2 + 2], si
	mov byte [si], 0
.havetwo:

	cmp	byte [si],0		;did user enter 2d file name?	v1.1
					; (should not be 0)		v1.1
	jnz	L2			; fname2 is non-blank, go check	for ext

	mov	si, [filerecords.1]	;source is FNAME1		v1.1
	mov	di,si	;offset FNAME1	;into DI for a scasb		v1.1
	mov	al,'.'
	cld
	repnz	scasb			; search fname1 for '.'
	not	cx			; cx has length of fname1 including '.'
	dec	cx			; don't copy the period
	mov	di, [filerecords.2]	; let Dest.  Index point to fname2 v1.1
					;SI (source) is already FNAME1	v1.1
	cld				; direction of movement = forward
	rep	movsb			; move chars from [SI] to [DI]
					;   CX has count (# chars to move)
	jmp	short L2a		; go add the ".COM"
;
L2:
	mov	di, [filerecords.2]	; check fname2 for extension
	mov dx, [filerecords.2 + 2]
	inc dx
	sub dx, di			; = length including NUL
	mov cx, dx

	mov	al,'.'
	repnz	scasb			; search for period in file name
	jz	L2b			; found the period, must have ext

	mov	ax, [filerecords.2]	;v1.1
	add	ax,dx	;fname2_len	;add in FNAME2's length		v1.1
	dec	ax			;adjust
	mov	di,ax			; di points to '\0' at end of file name
L2a:	mov	ax,'.C'	;432eh		; ".C" backwards		v1.1
	stosw
	mov	ax,'OM'	;4d4fh		;"OM" backwards			v1.1
	stosw
	mov word [filerecords.2 + 2], di
	mov al, 00h
	stosb
L2b:

displayfilenames:
	mov bp, filerecords
.loop:
	mov dx, [bp + 4]		; -> message
	mov ah, 09h
	int 21h

	cmp bp, filerecords.end		; at end ?
	jae .done			; yes -->

	mov dx, [bp]			; -> pathname
	mov cx, [bp + 2]		; -> behind pathname
	sub cx, dx			; = length
	jz .next
	mov bx, 1			; handle 1 = stdout
	mov ah, 40h
	int 21h				; write to handle
.next:
	add bp, 6			; -> next record
	jmp .loop

.done:


;****************************************************************************
;*	OPEN THE INPUT FILE
;****************************************************************************

	mov	dx, [filerecords.1]	;open input file
	mov	ax,3D00H		;open, read only
	int	21H
	jnc	L3			; opened ok
	 jmp	Open_Fail		; no file

L3:	mov	[handle1],ax		; save token for file

;****************************************************************************
;*	OPEN THE OUTPUT FILE
;****************************************************************************
	mov	dx, [filerecords.2]	;output file name
	xor	cx,cx			;normal attributes
	mov	ah,3CH			;create file
	int	21H
	jnc	L4			; created ok
	 jmp	Create_Fail		; create failed

L4:	mov	[handle2],ax		; save token for file

;****************************************************************************
;*	PROCESS THE EXE HEADER
;****************************************************************************
; first we read in the exe header, don't you think?

	mov	dx,buffer		;read buffer
	mov	cx,SIZE_OF_HEADER
	mov	bx,[handle1]
	mov	ah,3FH			;read from file/device
	int	21H
	jnc	L5			;read ok
	 jmp	Bad_Read		;read error

L5:
	cmp ax, cx
	je .full
	mov dx, msg.shortreadhead
	jmp Print_Error

.full:
	mov	bx,buffer	; use DS:BX to address buffer
	cmp	word [bx+exe_header.exe_sig1],'MZ'	;'MZ' signature?
	je	L7			;YEP, OK
	cmp	word [bx+exe_header.exe_sig1],'ZM'	; reverse is valid too
	je	L7			;YEP, OK
	 jmp	BadSig			;no, print error msg, exit	v1.1

L7:
	xor	ax,ax			;handy 0			v1.1

	cmp	[bx+exe_header.relo_ct],ax			;is relocatable count 0? v1.1
	je	L8				; is ok, no relocatable items
	 call	HasRelo				; oops, can't convert

L8:
	cmp	[bx+exe_header.xss],ax			;is stack segment 0?	v1.1
	je	L9				; is ok, no stack segment
	 call	Has_SS

L9:
	cmp	[bx+exe_header.xsp],ax			;is stack segment offset 0? v1.1
	je	L10				; is okay
	 call	Has_SS

L10:
	mov	ax,[bx+exe_header.xip]			; initial value for IP
	or	ax,ax				; should either be 0 or 100h v1.1
	je	L11
	cmp	ax,100h
	je	L11
	 call	Bad_IP

L11:	mov	[save_ip],ax		; save the IP for later use
;***************************************************************************
;* Compute offset of program image in module, and program size
;*
;* The program size is computed as follows;  it cannot exceed 64k bytes
;*     512 * (# of EXE pages -1 )
;*   + valid bytes in last EXE page
;*   - offset of program image in EXE file
;*
;* Note that if the IP is nonzero, we will skip the first
;* IP bytes of the program image, and copy IP bytes fewer
;* than the actual size
;*
;***************************************************************************

	mov	ax,[bx+exe_header.hdr_size]	; size of the program header
					; expressed in 16 byte paragraphs
	mov	cx,4			; no of times to shift
	xor dx, dx
.loop:
	shl ax, 1
	rcl dx, 1
	loop .loop			; times 16

	mov [code_start], ax		; save it
	mov [code_start + 2], dx	; save it

	mov	ax,[bx+exe_header.pages]		;nr of 512-byte pages		v1.1
	sub ax, 1
	jc header_error

L12:
	xor	dx, dx
	mov	cx, 9			; number of times to shift left
.loop:
	shl 	ax, 1
	rcl	dx, 1
	loop	.loop			; times 512

L13:
	mov	cx,[bx+exe_header.excess]		; no. of bytes in last non-full page
	cmp cx, 512
	jbe .
	mov cx, 512
.:
	add	ax,cx			; add to result from above
	adc	dx, 0
	sub	ax, [code_start]	; subtract the code start address
	sbb	dx, [code_start + 2]
	jc header_error
	mov	[code_size],ax		; save it for later
	mov	[code_size + 2], dx	; save it for later

	mov	di,code_size_str	;where to write the Ascii chars	v1.1
	call	BinToStr		;convert code size		v1.1
	mov	dx,code_size_msg	;'Code size = xxxx'		v1.1
	mov	ah,9			;display msg
	int	21H

	mov	ax,[code_start]		;convert code start address	v1.1
	mov	dx,[code_start + 2]	;convert code start address	v1.1
	mov	di,code_start_str ;where to write the Ascii chars v1.1
	call	BinToStr
	mov	dx,code_start_msg	;'Code start = xxxx'	v1.1
	mov	ah,9
	int	21H

	mov	ax,[save_ip]		;program's IP
	xor dx, dx
	mov	di,ip_str	;where to write the Ascii chars	v1.1
	call	BinToStr
	mov	dx,ip_msg	;'Initial IP = xxxx'		v1.1
	mov	ah,9
	int	21H

;****************************************************************************
;*	MOVE FILE POINTER TO START OF CODE
;****************************************************************************
;* Add the initial IP to the code start address.  This will give
;* us the file offset, which is the location in the EXE file that we
;* will start copying from
;*
	mov ax, [code_start]
	mov dx, [code_start + 2]
	add ax, [save_ip]		;add in program's IP		v1.1
	adc dx, 0
	mov cx, dx
	xchg ax, dx			;DX needs it as lower part	v1.1
					;of file offset			v1.1

	mov	bx,[handle1]		; handle of input file
	mov	ax,4200h		; move file pointer		v1.1
					;(from start)
	int	21h
	jc Seek_Error

L14:
;****************************************************************************
;*	COPY THE CODE TO THE OUTPUT (.COM) FILE
;****************************************************************************
; reduce the code_size by the size of the IP
	mov ax, [code_size]		; reduce the code_size
	mov dx, [code_size + 2]
	sub ax, [save_ip]		; by the IP size		v1.1
	sbb dx, 0
	jc header_error
	mov [code_size], ax		; store it away
	mov [code_size + 2], dx		; store it away

	mov	di,actual_code_size_str	;v1.1
	call	BinToStr		;convert code size

	mov	dx,actual_code_size_msg	;'Size of .COM file = xxxx' v1.1
	mov	ah,9
	int	21H

	rol byte [big_enable], 1
	jc .Next
	cmp word [code_size + 2], 0
.Too_Big_NZ:
	jne Too_Big
	cmp word [code_size], 64 * 1024 - 512
	ja .Too_Big_NZ
.Next:

Next:					; process next record
	mov	ax,RECSIZE
	cmp word [code_size + 2], 0
	jne L15
	cmp	ax,[code_size]		; compare code_size to RECSIZE
	jbe	L15			; if ax < code_size, use ax as is
	 mov	ax,[code_size]		;  else use size of remaining code
L15:	mov	cx,ax			; # of bytes to read
	mov	bx,[handle1]		;  input file
	mov	dx,buffer	; where to put it
	mov	ax,3f00h		; read from file/device
	int	21h
	jc Bad_Read

L16:
	cmp cx, ax
	je .fullread
	rol byte [eof_enable], 1
	jnc .error
	test ax, ax
	jnz .partialread
	jmp Done_j

.error:
	mov dx, msg.shortreaddata
	jmp Print_Error

.partialread:
.fullread:
	mov	cx,ax			; nr bytes to write		v1.1
	mov	bx,[handle2]		;  output file
	mov	dx,buffer	; addr of what we are about to write
	mov	ah,40h			; write to file/dev
	int	21h
	jnc	L18			; carry flag not set, no error on write
	 jmp	Bad_Write		; jump if write error

L18:	cmp	ax,cx	;wsize		; AX has # of bytes actually written
	je	L19			; if we wrote all of the bytes
					;    that we wanted to, then keep on
	 jmp	Disk_Full		; did NOT write all of the bytes
					; that I wanted to, disk must be full
L19:	sub word [code_size], ax
	sbb word [code_size + 2], 0
.Next_NZ:
	jne Next			; if upper word is nonzero -->
	cmp word [code_size], 0
	jne .Next_NZ			; if (only) lower word is nonzero -->
Done_j:
	jmp Done


;****************************************************************************
;*	ERROR MESSAGES
;****************************************************************************

wrongswitch:	mov dx, msg.wrongswitch
		jmp	short Print_Error
writehelp:
No_File1:	mov	dx,no_file_msg1
		jmp	short Print_Error
Open_Fail:	mov	dx,open_fail_msg
		jmp	short Print_Error
Create_Fail:	mov	dx,create_fail_msg
		jmp	short Print_Error
Disk_Full:	mov	dx,disk_full_msg
		jmp	short Print_Error
Bad_Read:	mov	dx,bad_read_msg
		jmp	short Print_Error
Bad_Write:	mov	dx,bad_write_msg
		jmp	short Print_Error
Seek_Error:	mov	dx,seek_error_msg
		jmp	short Print_Error
BadSig:		mov	dx,badsig_msg
		jmp	short Print_Error
HasRelo:
	push dx
	push bx
	push ax
	mov bx, ignore_reloc
		mov	dx,hasrelo_msg
	jmp ignore_common

Has_SS:
	push dx
	push bx
	push ax
	mov bx, ignore_stack
		mov	dx,has_ss_msg
	jmp ignore_common

Bad_IP:
	xor ax, ax		; special case: pretend ip = 0
	push dx
	push bx
	push ax
	mov bx, ignore_ip
		mov	dx,bad_ip_msg
	; jmp ignore_common

ignore_common:
	rol byte [bx], 1
	jnc Print_Error
	mov ah, 09h
	int 21h
	pop ax
	pop bx
	pop dx
	retn

Too_Big:	mov	dx,too_big_msg
		jmp	short Print_Error

header_error:	mov	dx, msg.headererror

Print_Error:	mov	ah,9
		int	21h
		not	BYTE [errflag]		;turn error flag on		v1.1
					;(non-zero)

;****************************************************************************
;*	CLOSE INPUT AND OUTPUT FILES
;****************************************************************************

Done:	mov	ax,[handle1]		;input file handle
	or	ax,ax			; is the handle still zero,
					; as was initially? v1.1
	je	Get_Out			; yes, no files to close
	mov	ah,3eh			; close input file
	mov	bx,[handle1]
	int	21h

	mov	ax,[handle2]		;output file handle
	or	ax,ax			; is the handle still zero,
					; as was initially? v1.1
	je	Get_Out			; yes, no output file to close
	mov	ah,3eh			; close output file
	mov	bx,[handle2]
	int	21h

	cmp	BYTE [errflag],0		;any errors?			v1.1
	jz	Get_Out			; no
	 mov	dx, [filerecords.2]	; yes, delete the output file	v1.1
	 mov	ah,41H			;delete file
	 int	21H

;****************************************************************************
;*	EXIT WITH STATUS CODE
;****************************************************************************

Get_Out:
	mov	ax,4C00H		;terminate, errorlevel 0	v1.1
	int	21H


; Purpose   Converts integer to string
; Input     dx:ax = number to convert, di = near address for write	v1.1

BinToStr:

	mov bx, 10			; Divide by 10
	push bx				; terminator
; Convert and save on stack backwards

GetDigit:
	mov cx, ax
	xchg ax, dx
	xor dx, dx			; dx:ax = high word
	div bx				; ax = high word result, dx = remainder
	xchg ax, cx			; cx = high word result, dx:ax = lower
	div bx				; cx:ax = result
	push dx				; Save on stack
	mov dx, cx			; dx:ax = result
	or cx, ax			; = 0 if entire dword is 0
	jnz GetDigit

; Take off the stack and store forward

PutDigit:
.:
	pop ax				; Get character
	cmp al, 10
	jae .ret
	add al, '0'
	stosb				; Store it
	jmp .

.ret:
	retn


buffer: 	equ ($ - $$ + 256 + 15) & ~15
filenamesbuffer:equ	buffer + RECSIZE
stack:		equ	filenamesbuffer + 2 * PATHNAMESIZE
stacktop:	equ	stack + STACKSIZE
